@@ -1,3 +1,5 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+
 #include <config.h>
 #include "add-printer.h"
 
@@ -22,7 +24,6 @@
 #include <libgnomecups/gnome-cups-util.h>
 
 #include "druid-helper.h"
-#include "printerconf.h"
 #include "snmpinter.h"
 #include "util.h"
 
@@ -49,9 +50,9 @@ typedef struct {
 	char *vendor_and_model;
 } LocalPrinter;
 
-/* 
+/*
  * PPDs are taken from cups and filtered into the following data structures:
- * 
+ *
  * vendors (GHashTable *)
  *    "vendor1" -> ...
  *    "vendor2" -> ...
@@ -62,7 +63,7 @@ typedef struct {
  *                                  PPD *
  *                                  PPD *
  *                                  PPD *
- * 
+ *
  */
 
 /* Identifies a PPD retrieved from the cups server */
@@ -78,22 +79,22 @@ typedef struct
 
 
 static void
-gnome_cups_error_dialog (GtkWindow *window, 
+gnome_cups_error_dialog (GtkWidget *w,
 			 const char *prefix,
 			 GError *error)
 {
 	GtkWidget *dialog;
 	char *msg;
-	
+
 	msg = error ? error->message : _("Unknown Error");
 
-	dialog = gtk_message_dialog_new (window, 
-					 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL, 
+	dialog = gtk_message_dialog_new (GTK_WINDOW (w),
+					 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
 					 GTK_MESSAGE_ERROR,
 					 GTK_BUTTONS_OK,
 					 "%s: %s", prefix, error->message);
 	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog); 
+	gtk_widget_destroy (dialog);
 }
 
 static void
@@ -122,7 +123,7 @@ combine_vendor_and_model (const char *vendor,
 {
 	char *prefix;
 	char *ret;
-	
+
 	if (strstr (model, vendor) == model) {
 		return g_strdup (model);
 	} else {
@@ -134,13 +135,19 @@ combine_vendor_and_model (const char *vendor,
 }
 
 static char *
-parse_pconf (char *line)
+parse_network_detect (char *line)
 {
 	char *vendor_and_model;
 	char *vendor = NULL;
 	char *model = NULL;
 	char **fields;
 	char **p;
+	char *sep;
+
+	sep = strchr (line, '\n');
+	if (sep) {
+		*sep = '\0';
+	}
 
 	line = g_strstrip (line);
 	fields = g_strsplit (line, ";", -1);
@@ -149,20 +156,20 @@ parse_pconf (char *line)
 		char **pieces;
 		char *name;
 		char *value;
-		
+
 		pieces = g_strsplit (*p, "=", -1);
 		name = pieces[0];
 		value = pieces[1];
-		if (!name || !value) { 
+		if (!name || !value) {
 			g_strfreev (pieces);
 			continue;
 		}
-		
+
 		if (!strcmp (name, "vendor")) {
 			vendor = g_strdup (value);
 		} else if (!strcmp (name, "model")) {
 			model = g_strdup (value);
-		} 
+		}
 
 		g_strfreev (pieces);
 	}
@@ -173,7 +180,7 @@ parse_pconf (char *line)
 		g_free (model);
 		return NULL;
 	}
-	
+
 	vendor_and_model = combine_vendor_and_model (vendor, model);
 
 	g_free (vendor);
@@ -209,15 +216,15 @@ str_case_compare (const char *v1,
 	char *a;
 	char *b;
 	int ret;
-	
+
 	a = g_utf8_casefold (v1, -1);
 	b = g_utf8_casefold (v2, -1);
-	
+
 	ret = g_utf8_collate (a, b);
-	
+
 	g_free (a);
 	g_free (b);
-	
+
 	return ret;
 }
 
@@ -226,9 +233,9 @@ model_list_for_vendor (GHashTable *vendors,
 		       const char *vendor)
 {
 	GHashTable *models;
-	
+
 	models = g_hash_table_lookup (vendors, vendor);
-	
+
 	return g_list_sort (gnome_cups_hash_table_keys (models),
 			    (GCompareFunc)str_case_compare);
 }
@@ -236,7 +243,7 @@ model_list_for_vendor (GHashTable *vendors,
 static GList *
 vendor_list (GHashTable *vendors)
 {
-	return g_list_sort (gnome_cups_hash_table_keys (vendors), 
+	return g_list_sort (gnome_cups_hash_table_keys (vendors),
 			    (GCompareFunc)str_case_compare);
 }
 
@@ -247,7 +254,7 @@ remove_vendor (const char *vendor,
 {
         char *prefix;
         char *ret;
-	
+
 	prefix = g_strconcat (vendor, " ", NULL);
 	if (strstr (model, prefix) == model) {
 		ret = g_strstrip (g_strdup (model + strlen (prefix)));
@@ -255,18 +262,18 @@ remove_vendor (const char *vendor,
 		ret = g_strdup (model);
         }
         g_free (prefix);
-	
+
         return ret;
 }
 
 /* Return number of characters that match, or -1 if they are identical */
-static int 
+static int
 num_match (const char *a, const char *b)
 {
 	char *a1;
 	char *b1;
 	int i;
-	
+
 	a1 = g_utf8_casefold (a, -1);
 	b1 = g_utf8_casefold (b, -1);
 
@@ -298,7 +305,7 @@ get_detected_ppd (GladeXML *xml, const char *vendor_and_model)
 	char *model;
 	char *ppd_model;
 	int highest_match;
-	
+
 	if (!vendor_and_model) {
 		return NULL;
 	}
@@ -306,7 +313,7 @@ get_detected_ppd (GladeXML *xml, const char *vendor_and_model)
 	vendor_hash = g_object_get_data (G_OBJECT (xml), "vendors");
 
 	vendors = vendor_list (vendor_hash);
-	
+
 	vendor = NULL;
 	model = NULL;
 
@@ -324,17 +331,17 @@ get_detected_ppd (GladeXML *xml, const char *vendor_and_model)
 			model = NULL;
 		}
 	}
-	
+
 	g_list_free (vendors);
-	
+
 	if (!vendor) {
 		return NULL;
 	}
 
-	/* Now we have a valid make and model.  Find the closest 
+	/* Now we have a valid make and model.  Find the closest
 	 * ppd */
 	models = model_list_for_vendor (vendor_hash, vendor);
-	
+
 	ppd_model = NULL;
 	highest_match = 0;
 	for (l = models; l != NULL; l = l->next) {
@@ -349,7 +356,7 @@ get_detected_ppd (GladeXML *xml, const char *vendor_and_model)
 			ppd_model = l->data;
 			break;
 		}
-		
+
 		if (match > highest_match) {
 			ppd_model = l->data;
 			highest_match = match;
@@ -368,18 +375,18 @@ get_detected_ppd (GladeXML *xml, const char *vendor_and_model)
 
 	g_free (vendor);
 	g_free (model);
-	
+
 	if (!drivers) {
 		return NULL;
 	}
-	
+
 	return drivers->data;
 }
 
 static void
 set_selected_uri (GladeXML *xml, const char *uri)
 {
-	g_object_set_data_full (G_OBJECT (xml), "selected_uri", 
+	g_object_set_data_full (G_OBJECT (xml), "selected_uri",
 				g_strdup (uri), g_free);
 }
 
@@ -387,10 +394,10 @@ static char *
 get_selected_uri (GladeXML *xml)
 {
 	char *str;
-	
+
 	str = g_object_get_data (G_OBJECT (xml), "selected_uri");
 	g_return_val_if_fail (str != NULL, NULL);
-	
+
 	return g_strdup (str);
 }
 
@@ -405,10 +412,10 @@ static char *
 get_selected_location (GladeXML *xml)
 {
 	char *str;
-	
+
 	str = g_object_get_data (G_OBJECT (xml), "selected_location");
 	g_return_val_if_fail (str != NULL, NULL);
-	
+
 	return g_strdup (str);
 }
 
@@ -422,9 +429,9 @@ select_item (GtkWidget *option_menu, const char *name)
 	if (!name) {
 		return;
 	}
-	
+
 	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (option_menu));
-		
+
 	i = 0;
 	for (l = GTK_MENU_SHELL (menu)->children; l != NULL; l = l->next) {
 		int compare;
@@ -432,7 +439,7 @@ select_item (GtkWidget *option_menu, const char *name)
 		char *item_name = g_object_get_data (G_OBJECT (item), "name");
 		compare = str_case_compare (item_name, name);
 		if (compare >= 0) {
-			gtk_option_menu_set_history 
+			gtk_option_menu_set_history
 				(GTK_OPTION_MENU (option_menu), i);
 			break;
 		}
@@ -453,7 +460,7 @@ scroll_to_iter (GtkTreeView *view, GtkTreeModel *model, GtkTreeIter *iter)
 	GtkTreePath *path;
 
 	path = gtk_tree_model_get_path (model, iter);
-	
+
 	gtk_tree_view_scroll_to_cell (view,
 				      path,
 				      NULL,
@@ -470,31 +477,31 @@ select_model (GladeXML *xml, const char *model_name)
 	GtkTreeSelection *selection;
 	GtkTreeIter iter;
 	gboolean success;
-	
+
 	view = GTK_TREE_VIEW (glade_xml_get_widget (xml, "model_treeview"));
 	model = gtk_tree_view_get_model (view);
 	selection = gtk_tree_view_get_selection (view);
-	
+
 	success = gtk_tree_model_get_iter_first (model, &iter);
-	
+
 	gtk_tree_selection_select_iter (selection, &iter);
 	scroll_to_iter (view, model, &iter);
-	
+
 	while (success) {
 		int compare;
 		char *item_name;
 
 		gtk_tree_model_get (model, &iter, 0, &item_name, -1);
 		compare = str_case_compare (item_name, model_name);
-		if (compare >= 0) {			
+		if (compare >= 0) {
 			gtk_tree_selection_select_iter (selection, &iter);
 			scroll_to_iter (view, model, &iter);
 			break;
 		}
 		gtk_tree_model_iter_next (model, &iter);
 	}
-				      
-	
+
+
 }
 
 static void
@@ -525,9 +532,9 @@ static GtkWidget *
 local_or_remote_next (GladeXML *xml)
 {
 	GtkWidget *local_radio;
-	
+
 	local_radio = glade_xml_get_widget (xml, "local_radio");
-	
+
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (local_radio))) {
 		return glade_xml_get_widget (xml, "local_location_page");
 	} else {
@@ -541,7 +548,7 @@ static char *
 get_local_manual_uri (GladeXML *xml)
 {
 	GtkWidget *item;
-	
+
 	item = option_menu_get_selected_item (xml, "local_port_option_menu");
 	if (item) {
 		LocalPrinter *desc;
@@ -555,7 +562,7 @@ static char *
 get_local_manual_location (GladeXML *xml)
 {
 	GtkWidget *item;
-	
+
 	item = option_menu_get_selected_item (xml, "local_port_option_menu");
 	if (item) {
 		LocalPrinter *desc;
@@ -575,7 +582,7 @@ setup_port_option_menu (GladeXML *xml)
 
 	option_menu = glade_xml_get_widget (xml, "local_port_option_menu");
 	menu = gtk_menu_new ();
-	
+
 	devices = g_object_get_data (G_OBJECT (xml), "local-devices");
 
 	for (; devices != NULL; devices = devices->next) {
@@ -584,15 +591,15 @@ setup_port_option_menu (GladeXML *xml)
 		char *label;
 
 		if (desc->vendor_and_model) {
-			label = g_strdup_printf (_("%s (%s)"), 
+			label = g_strdup_printf (_("%s (%s)"),
 						 desc->label,
 						 desc->vendor_and_model);
 		} else {
 			label = g_strdup (desc->label);
 		}
-		
+
 		item = gtk_menu_item_new_with_label (label);
-		g_object_set_data (G_OBJECT (item), 
+		g_object_set_data (G_OBJECT (item),
 				   "printer-description", desc);
 		gtk_widget_show (item);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -610,7 +617,7 @@ update_local_location_sensitivities (GtkWidget *widget,
 
 	view = glade_xml_get_widget (xml, "local_detected_view");
 	option_menu = glade_xml_get_widget (xml, "local_port_option_menu");
-	
+
 	if (toggle_button_is_active (xml, "local_use_detected_radio")) {
 		gtk_widget_set_sensitive (view, TRUE);
 		gtk_widget_set_sensitive (option_menu, FALSE);
@@ -649,10 +656,10 @@ get_local_devices (void)
 					desc = g_new0 (LocalPrinter, 1);
 				}
 				desc = g_new0 (LocalPrinter, 1);
-				
+
 				g_free (device_class);
 				device_class = NULL;
-				
+
 				continue;
 			}
 
@@ -670,7 +677,7 @@ get_local_devices (void)
 				desc->vendor_and_model = g_strdup (attr->values[0].string.text);
 			}
 		}
-		
+
 		if (device_class
 		    && strcmp (device_class, "network")
 		    && desc->label
@@ -705,12 +712,12 @@ local_location_setup (GladeXML *xml)
 
 
 	devices = get_local_devices ();
-	g_object_set_data_full (G_OBJECT (xml), "local-devices", 
+	g_object_set_data_full (G_OBJECT (xml), "local-devices",
 				devices,
 				(GDestroyNotify)local_printer_list_free);
-	
+
 	tree_view = GTK_TREE_VIEW (glade_xml_get_widget (xml, "local_detected_view"));
-	
+
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (_("Printer"),
 							   renderer,
@@ -719,7 +726,7 @@ local_location_setup (GladeXML *xml)
 							   NULL);
 	gtk_tree_view_append_column (tree_view, column);
 
-	list_store = gtk_list_store_new (LAST_LOCAL_DETECTED, 
+	list_store = gtk_list_store_new (LAST_LOCAL_DETECTED,
 					 G_TYPE_STRING, G_TYPE_POINTER);
 	gtk_tree_view_set_model (tree_view, GTK_TREE_MODEL (list_store));
 
@@ -732,42 +739,42 @@ local_location_setup (GladeXML *xml)
 		     * regardless of what's there */
 		    && strcmp (desc->vendor_and_model, "EPSON")
 		    && strcmp (desc->vendor_and_model, "CANON")) {
-			GtkTreeIter iter;		
-			
+			GtkTreeIter iter;
+
 			gtk_list_store_append (list_store, &iter);
-			gtk_list_store_set (list_store, &iter, 
-					    0, desc->vendor_and_model, 
-					    1, desc, 
+			gtk_list_store_set (list_store, &iter,
+					    0, desc->vendor_and_model,
+					    1, desc,
 					    -1);
 			if (!num_detected) {
 				gtk_tree_selection_select_iter (selection, &iter);
 			}
-			
+
 			num_detected++;
 		}
-		
+
 
 	}
 
 	if (!num_detected) {
 		GtkTreeIter iter;
 		gtk_list_store_append (list_store, &iter);
-		gtk_list_store_set (list_store, &iter, 
-				    0, _("<i>No printers detected</i>"), 
-				    1, NULL, 
+		gtk_list_store_set (list_store, &iter,
+				    0, _("<i>No printers detected</i>"),
+				    1, NULL,
 				    -1);
 
 		gtk_tree_selection_set_mode (selection, GTK_SELECTION_NONE);
 	}
 
 	setup_port_option_menu (xml);
-	
+
 	widget = glade_xml_get_widget (xml, "local_use_detected_radio");
-	g_signal_connect (widget, "toggled", 
+	g_signal_connect (widget, "toggled",
 			  G_CALLBACK (update_local_location_sensitivities),
 			  xml);
 	widget = glade_xml_get_widget (xml, "local_specify_port_radio");
-	g_signal_connect (widget, "toggled", 
+	g_signal_connect (widget, "toggled",
 			  G_CALLBACK (update_local_location_sensitivities),
 			  xml);
 
@@ -789,7 +796,7 @@ local_location_back (GladeXML *xml)
 	return glade_xml_get_widget (xml, "local_or_remote_page");
 }
 
-static GtkWidget * 
+static GtkWidget *
 local_location_next (GladeXML *xml)
 {
 	GtkTreeView *tree_view;
@@ -797,23 +804,23 @@ local_location_next (GladeXML *xml)
 	GtkTreeSelection *selection;
 	GtkTreeIter iter;
 	gboolean use_detected;
-	
+
 	tree_view = GTK_TREE_VIEW (glade_xml_get_widget (xml, "local_detected_view"));
 	selection = gtk_tree_view_get_selection (tree_view);
 	list_store = GTK_LIST_STORE (gtk_tree_view_get_model (tree_view));
 
 	use_detected = !toggle_button_is_active (xml, "local_specify_port_radio");
-	
+
 	select_detected_ppd (xml, NULL);
 
 	if (use_detected) {
 		if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
 			char *location;
 			LocalPrinter *desc;
-			gtk_tree_model_get (GTK_TREE_MODEL (list_store), 
+			gtk_tree_model_get (GTK_TREE_MODEL (list_store),
 					    &iter,
 					    1, &desc, -1);
-			location = g_strdup_printf (_("Local printer on %s"), 
+			location = g_strdup_printf (_("Local printer on %s"),
 						       desc->location);
 			select_detected_ppd (xml, desc->vendor_and_model);
 			set_selected_uri (xml, desc->uri);
@@ -826,7 +833,7 @@ local_location_next (GladeXML *xml)
 	} else {
 		char *uri;
 		char *location;
-		
+
 		uri = get_local_manual_uri (xml);
 		if (!uri) {
 			g_warning ("no selected port");
@@ -840,7 +847,7 @@ local_location_next (GladeXML *xml)
 		g_free (uri);
 		g_free (location);
 	}
-	
+
 	return glade_xml_get_widget (xml, "details_page");
 }
 
@@ -851,7 +858,7 @@ static NetworkType
 get_manual_network_type (GladeXML *xml)
 {
 	int index;
-	
+
 	index = gtk_option_menu_get_history (GTK_OPTION_MENU (glade_xml_get_widget (xml, "network_type_option")));
 
 	return (NetworkType)index;
@@ -863,20 +870,20 @@ get_manual_lpd_uri (GladeXML *xml)
 	char *ret = NULL;
 	char *host;
 	char *queue;
-	
+
 	host = g_strdup (ENTRY_TEXT (xml, "lpd_host_entry"));
 	queue = g_strdup (ENTRY_TEXT (xml, "lpd_queue_entry"));
-	
+
 	host = g_strstrip (host);
 	queue = g_strstrip (queue);
-	
+
 	if (host[0]) {
 		ret = g_strdup_printf ("lpd://%s/%s", host, queue);
 	}
-	
+
 	g_free (host);
 	g_free (queue);
-	
+
 	return ret;
 }
 
@@ -888,16 +895,16 @@ get_manual_smb_uri (GladeXML *xml)
 	char *printer;
 	char *username;
 	char *password;
-	
+
 	host = g_strdup (ENTRY_TEXT (xml, "smb_host_entry"));
 	printer = g_strdup (ENTRY_TEXT (xml, "smb_printer_entry"));
 	username = g_strdup (ENTRY_TEXT (xml, "smb_username_entry"));
 	password = g_strdup (ENTRY_TEXT (xml, "smb_password_entry"));
-	
+
 	host = g_strstrip (host);
 	printer = g_strstrip (printer);
 	username = g_strstrip (username);
-	
+
 	if (host[0] && printer[0]) {
 		if (username[0]) {
 			ret = g_strdup_printf ("smb://%s:%s@%s/%s",
@@ -925,17 +932,17 @@ get_manual_ipp_uri (GladeXML *xml)
 {
 	char *ret = NULL;
 	char *uri;
-	
+
 	uri = g_strdup (ENTRY_TEXT (xml, "ipp_uri_entry"));
-	
+
 	uri = g_strstrip (uri);
-	
+
 	if (uri[0]) {
 		ret = g_strdup (uri);
 	}
-	
+
 	g_free (uri);
-	
+
 	return ret;
 }
 
@@ -945,10 +952,10 @@ get_manual_hp_uri (GladeXML *xml)
 	char *ret = NULL;
 	char *host;
 	char *port;
-	
+
 	host = g_strdup (ENTRY_TEXT (xml, "hp_host_entry"));
 	port = g_strdup (ENTRY_TEXT (xml, "hp_port_entry"));
-	
+
 	host = g_strstrip (host);
 	port = g_strstrip (port);
 
@@ -956,14 +963,14 @@ get_manual_hp_uri (GladeXML *xml)
 		g_free (port);
 		port = g_strdup ("9100");
 	}
-	
+
 	if (host[0]) {
 		ret = g_strdup_printf ("socket://%s:%s", host, port);
 	}
-	
+
 	g_free (host);
 	g_free (port);
-	
+
 	return ret;
 }
 
@@ -972,7 +979,7 @@ static char *
 get_manual_network_uri (GladeXML *xml)
 {
 	char *ret;
-	
+
 	ret = NULL;
 
 	switch (get_manual_network_type (xml)) {
@@ -999,7 +1006,7 @@ static char *
 get_manual_network_location (GladeXML *xml)
 {
 	char *ret;
-	
+
 	ret = NULL;
 
 	switch (get_manual_network_type (xml)) {
@@ -1007,13 +1014,13 @@ get_manual_network_location (GladeXML *xml)
 	{
 		char *host;
 		char *queue;
-		
+
 		host = g_strdup (ENTRY_TEXT (xml, "lpd_host_entry"));
 		queue = g_strdup (ENTRY_TEXT (xml, "lpd_queue_entry"));
-		
+
 		host = g_strstrip (host);
 		queue = g_strstrip (queue);
-		
+
 		if (host[0]) {
 			if (queue[0]) {
 				/* queue, host */
@@ -1021,7 +1028,7 @@ get_manual_network_location (GladeXML *xml)
 			} else {
 				/* host */
 				ret = g_strdup_printf (_("UNIX printer on %s"), host);
-			}	
+			}
 		}
 		break;
 	}
@@ -1029,13 +1036,13 @@ get_manual_network_location (GladeXML *xml)
 	{
 		char *host;
 		char *printer;
-		
+
 		host = g_strdup (ENTRY_TEXT (xml, "smb_host_entry"));
 		printer = g_strdup (ENTRY_TEXT (xml, "smb_printer_entry"));
-		
+
 		host = g_strstrip (host);
 		printer = g_strstrip (printer);
-		
+
 		if (host[0] && printer[0]) {
 			/* printer, host */
 			ret = g_strdup_printf (_("Windows Printer %s on %s"), printer, host);
@@ -1045,11 +1052,11 @@ get_manual_network_location (GladeXML *xml)
 	case NETWORK_TYPE_IPP :
 	{
 		char *uri;
-		
+
 		uri = g_strdup (ENTRY_TEXT (xml, "ipp_uri_entry"));
-		
+
 		uri = g_strstrip (uri);
-		
+
 		if (uri[0]) {
 			/* uri */
 			ret = g_strdup_printf (_("IPP Printer at %s"), uri);
@@ -1067,17 +1074,17 @@ static char *
 lpd_get_vendor_and_model (const char *host)
 {
 	char *ret;
-	char **detected_printers;
+	char *detected;
 	int retval;
-	
-	detected_printers = pconf_detect_printer (PCONF_DETECT_NETWORK, 
-						  (char*)host,
-						  &retval);
 
-	ret = NULL;
-	if (detected_printers) {
-		ret = parse_pconf (detected_printers[0]);
+	detected = g_strdup (get_snmp_printers ((char*)host, &retval));
+
+	if (detected) {
+		ret = parse_network_detect (detected);
+	} else {
+		ret = NULL;
 	}
+	g_free (detected);
 
 	return ret;
 }
@@ -1088,7 +1095,7 @@ get_manual_network_vendor_and_model (GladeXML *xml)
 	char *ret;
 	char *host;
 	host = NULL;
-	
+
 	switch (get_manual_network_type (xml)) {
 	case NETWORK_TYPE_LPD :
 		host = g_strdup (ENTRY_TEXT (xml, "lpd_host_entry"));
@@ -1111,7 +1118,7 @@ network_type_changed_cb (GtkOptionMenu *option_menu,
 {
 	int index;
 	GtkWidget *notebook;
-	
+
 	index = gtk_option_menu_get_history (option_menu);
 	notebook = glade_xml_get_widget (xml, "network_info_notebook");
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), index);
@@ -1121,7 +1128,7 @@ static void
 network_location_setup (GladeXML *xml)
 {
 	GtkWidget *option_menu;
-	
+
 	druid_watch_sensitivity_widget (xml, "network_type_option");
 	druid_watch_sensitivity_widget (xml, "lpd_host_entry");
 	druid_watch_sensitivity_widget (xml, "smb_username_entry");
@@ -1143,11 +1150,11 @@ network_location_sensitivity (GladeXML *xml, gboolean *back, gboolean *next)
 	char *uri;
 
 	*back = TRUE;
-	
+
 	uri = get_manual_network_uri (xml);
-	
+
 	*next = (uri != NULL);
-	
+
 	g_free (uri);
 }
 
@@ -1157,11 +1164,11 @@ network_location_back (GladeXML *xml)
 	return glade_xml_get_widget (xml, "local_or_remote_page");
 }
 
-static GtkWidget * 
+static GtkWidget *
 network_location_next (GladeXML *xml)
 {
 	char *uri;
-	
+
 	uri = get_manual_network_uri (xml);
 	if (!uri) {
 		return NULL;
@@ -1184,7 +1191,7 @@ network_location_next (GladeXML *xml)
 			select_detected_ppd (xml, NULL);
 		}
 		return glade_xml_get_widget (xml, "details_page");
-	}	
+	}
 }
 
 static ipp_t *
@@ -1192,13 +1199,13 @@ ppd_request (void)
 {
 	ipp_t *response;
 	ipp_t *request;
-	char *attributes[] = { "ppd-name", 
-			       "ppd-make", 
+	char *attributes[] = { "ppd-name",
+			       "ppd-make",
 			       "ppd-make-and-model",
 			       "ppd-natural-language" };
 
 	request = gnome_cups_request_new_for_printer (CUPS_GET_PPDS, NULL);
-	gnome_cups_request_add_requested_attributes (request, 
+	gnome_cups_request_add_requested_attributes (request,
 						     IPP_TAG_PRINTER,
 						     G_N_ELEMENTS (attributes),
 						     attributes);
@@ -1210,7 +1217,7 @@ ppd_request (void)
 
 static void
 destroy_ppd_id (PPD *ppd)
-{					
+{
 	g_free (ppd->name);
 	g_free (ppd->vendor);
 	g_free (ppd->vendor_and_model);
@@ -1224,7 +1231,7 @@ static void
 destroy_ppd_id_list (GList *list)
 {
 	GList *l;
-	
+
 	for (l = list; l != NULL; l = l->next) {
 		destroy_ppd_id (l->data);
 	}
@@ -1232,7 +1239,7 @@ destroy_ppd_id_list (GList *list)
 }
 
 static void
-parse_ppd_vendor_and_model (const char *vendor, 
+parse_ppd_vendor_and_model (const char *vendor,
 			    const char *vendor_and_model,
 			    char **model,
 			    char **driver)
@@ -1261,31 +1268,31 @@ parse_ppd_vendor_and_model (const char *vendor,
 
 static GList *
 get_ppds (void)
-{	
+{
 	ipp_attribute_t *attr;
 	ipp_t *response;
 	PPD *ppd = NULL;
 	GList *ppd_ids;
 
 	response = ppd_request ();
-	
+
 	ppd_ids = NULL;
 	ppd = g_new0 (PPD, 1);
 
 	if (response) {
 		for (attr = response->attrs; attr != NULL; attr = attr->next) {
 			if (!attr->name) {
-				if (ppd->name 
-				    && ppd->vendor 
+				if (ppd->name
+				    && ppd->vendor
 				    && ppd->vendor_and_model
 				    && ppd->language) {
-					parse_ppd_vendor_and_model 
+					parse_ppd_vendor_and_model
 						(ppd->vendor,
 						 ppd->vendor_and_model,
 						 &ppd->model,
 						 &ppd->driver);
-					
-					ppd_ids = g_list_prepend (ppd_ids, 
+
+					ppd_ids = g_list_prepend (ppd_ids,
 								  ppd);
 				} else {
 					destroy_ppd_id (ppd);
@@ -1300,11 +1307,11 @@ get_ppds (void)
 				g_assert (ppd->name == NULL);
 				ppd->name = g_strdup (attr->values[0].string.text);
 			} else if (!strcmp (attr->name, "ppd-make")) {
-				/* FIXME */ 
+				/* FIXME */
 				g_assert (ppd->vendor == NULL);
 				ppd->vendor = g_strdup (attr->values[0].string.text);
 			} else if (!strcmp (attr->name, "ppd-make-and-model")) {
-				/* FIXME */ 
+				/* FIXME */
 				g_assert (ppd->vendor_and_model == NULL);
 				ppd->vendor_and_model = g_strdup (attr->values[0].string.text);
 			} else if (!strcmp (attr->name, "ppd-natural-language")) {
@@ -1321,7 +1328,7 @@ get_ppds (void)
 }
 
 
-static int 
+static int
 compare_models_by_driver (gconstpointer a, gconstpointer b)
 {
 	const PPD *a1 = a;
@@ -1334,7 +1341,7 @@ str_case_hash (gconstpointer key)
 {
 	char *str;
 	guint ret;
-	
+
 	str = g_ascii_strup (key, -1);
 	ret = g_str_hash (str);
 	g_free (str);
@@ -1342,10 +1349,10 @@ str_case_hash (gconstpointer key)
 	return ret;
 }
 
-static gboolean 
+static gboolean
 str_case_equal (gconstpointer v1,
 		gconstpointer v2)
-{	
+{
 	return str_case_compare (v1, v2) == 0;
 }
 
@@ -1354,18 +1361,18 @@ categorize_ppd_ids (GList *ppd_ids)
 {
 	GHashTable *models_for_vendors;
 	GList *l;
-	
-	models_for_vendors = g_hash_table_new_full (str_case_hash, 
+
+	models_for_vendors = g_hash_table_new_full (str_case_hash,
 						    str_case_equal,
-						    NULL, 
+						    NULL,
 						    (GDestroyNotify)g_hash_table_destroy);
-	
+
 	for (l = ppd_ids; l != NULL; l = l->next) {
 		GHashTable *models;
 		GList *drivers;
 		PPD *ppd = l->data;
 
-		models = g_hash_table_lookup (models_for_vendors, 
+		models = g_hash_table_lookup (models_for_vendors,
 					      ppd->vendor);
 		if (!models) {
 			models = g_hash_table_new_full
@@ -1375,7 +1382,7 @@ categorize_ppd_ids (GList *ppd_ids)
 					     ppd->vendor,
 					     models);
 		}
-		
+
 		drivers = g_hash_table_lookup (models, ppd->model);
 		drivers = g_list_insert_sorted (drivers, ppd, compare_models_by_driver);
 		g_hash_table_steal (models, ppd->model);
@@ -1405,10 +1412,10 @@ get_selected_model (GladeXML *xml)
 	GtkTreeSelection *selection;
 	GtkTreeIter iter;
 	GtkTreeModel *model;
-	
+
 	tree_view = glade_xml_get_widget (xml, "model_treeview");
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
-	
+
 	if (gtk_tree_selection_get_selected (selection,
 					     &model,
 					     &iter)) {
@@ -1420,12 +1427,12 @@ get_selected_model (GladeXML *xml)
 	}
 }
 
-	
+
 static PPD *
 get_selected_ppd (GladeXML *xml)
 {
 	GtkWidget *item;
-	
+
 	item = option_menu_get_selected_item (xml, "driver_option_menu");
 	if (item) {
 		PPD *ppd = g_object_get_data (G_OBJECT (item), "selected_ppd");
@@ -1448,10 +1455,10 @@ populate_drivers_from_list (GladeXML *xml, GList *drivers)
 		PPD *ppd = drivers->data;
 		GtkWidget *item;
 		char *label;
-		
-		label = g_strdup_printf (_("%s (%s)"), 
-					 ppd->driver, 
-					 ppd->language);		
+
+		label = g_strdup_printf (_("%s (%s)"),
+					 ppd->driver,
+					 ppd->language);
 		item = gtk_menu_item_new_with_label (label);
 		gtk_widget_show (item);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -1464,36 +1471,28 @@ populate_drivers_from_list (GladeXML *xml, GList *drivers)
 static void
 populate_driver_option_menu (GladeXML *xml)
 {
-	char *vendor;
-	char *model;
-	GtkWidget *drivers;
-		
-	drivers = glade_xml_get_widget (xml, "driver_option_menu");
-	
-	vendor = get_selected_vendor (xml);
-	model = get_selected_model (xml);
+	char	   *vendor  = get_selected_vendor (xml);
+	char	   *model   = get_selected_model (xml);
+	GtkWidget  *drivers = glade_xml_get_widget (xml, "driver_option_menu");
+	GHashTable *models  = NULL;
 
+	g_warning ("populate driver");
 	if (vendor && model) {
-		GHashTable *vendors = 
-			g_object_get_data (G_OBJECT (xml), "vendors");
-		GHashTable *models;
+		GHashTable *vendors = g_object_get_data (
+			G_OBJECT (xml), "vendors");
+		models = g_hash_table_lookup (vendors, vendor);
+	} else
+		gtk_widget_set_sensitive (drivers, FALSE);
 
-		gtk_widget_set_sensitive (drivers, TRUE);
-		
-		models = g_hash_table_lookup (vendors,
-					      vendor);
-		if (models) {
-			GList *drivers;
-			drivers = g_hash_table_lookup (models, model);
-			
-			populate_drivers_from_list (xml, drivers);
-		}
-	} else {
-		gtk_widget_set_sensitive (drivers, FALSE);		
-	}
+	gtk_widget_set_sensitive (drivers, models != NULL);
+	if (models != NULL)
+		populate_drivers_from_list (xml,
+			g_hash_table_lookup (models, model));
+	else
+		gtk_option_menu_remove_menu (GTK_OPTION_MENU (drivers));
 
 	g_free (vendor);
-	g_free (model);	
+	g_free (model);
 }
 
 static void
@@ -1531,14 +1530,6 @@ populate_model_list (GladeXML *xml)
 	g_free (vendor);
 }
 
-static void
-model_selection_changed_cb (GtkTreeSelection *selection,
-			    gpointer user_data)
-{
-	GladeXML *xml = GLADE_XML (user_data);
-	populate_driver_option_menu (xml);
-}
-
 static void
 vendor_option_menu_changed_cb (GtkWidget *option_menu,
 			       gpointer user_data)
@@ -1587,7 +1578,7 @@ details_setup (GladeXML *xml)
 	GtkCellRenderer *renderer;
 	GtkListStore *store;
 	GtkTreeSelection *selection;
-	
+
 	option_menu = glade_xml_get_widget (xml, "vendor_option_menu");
 	g_signal_connect (option_menu, "changed",
 			  G_CALLBACK (vendor_option_menu_changed_cb),
@@ -1603,25 +1594,26 @@ details_setup (GladeXML *xml)
 							   NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
 
-	gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), 
+	gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view),
 				 GTK_TREE_MODEL (store));
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
-	g_signal_connect (selection, "changed",
-			  G_CALLBACK (model_selection_changed_cb),
-			  xml);
-	
+	g_signal_connect_swapped (selection,
+		"changed",
+		G_CALLBACK (populate_driver_option_menu), xml);
+
+	druid_watch_sensitivity_widget (xml, "model_treeview");
 	druid_watch_sensitivity_widget (xml, "driver_option_menu");
 
-	populate_vendor_option_menu (xml);	
+	populate_vendor_option_menu (xml);
 }
 
 static void
 details_sensitivity (GladeXML *xml, gboolean *back, gboolean *next)
-{	
+{
 	*back = TRUE;
-	
+
 	if (get_selected_ppd (xml) != NULL) {
 		*next = TRUE;
 	} else {
@@ -1629,7 +1621,7 @@ details_sensitivity (GladeXML *xml, gboolean *back, gboolean *next)
 	}
 }
 
-static GtkWidget * 
+static GtkWidget *
 details_back (GladeXML *xml)
 {
 	if (toggle_button_is_active (xml, "local_radio")) {
@@ -1639,11 +1631,11 @@ details_back (GladeXML *xml)
 	}
 }
 
-static GtkWidget * 
+static GtkWidget *
 details_next (GladeXML *xml)
 {
 	PPD *ppd;
-	
+
 	ppd = get_selected_ppd (xml);
 	if (ppd) {
 		return glade_xml_get_widget (xml, "name_page");
@@ -1663,10 +1655,10 @@ setup_ppds (GladeXML *xml)
 				"ppds",
 				ppds,
 				(GDestroyNotify)g_list_free);
-	
+
 	vendors = categorize_ppd_ids (ppds);
-	g_object_set_data_full (G_OBJECT (xml), 
-				"vendors", 
+	g_object_set_data_full (G_OBJECT (xml),
+				"vendors",
 				vendors,
 				(GDestroyNotify)g_hash_table_destroy);
 }
@@ -1696,15 +1688,15 @@ name_next (GladeXML *xml)
 	char *c;
 	char *text;
 	gboolean valid_name;
-	
+
 	text = g_strdup (gtk_entry_get_text (GTK_ENTRY (glade_xml_get_widget (xml, "name_entry"))));
-	
+
 	if (!text) {
 		return NULL;
 	}
 
 	text = g_strstrip (text);
-	
+
 	valid_name = TRUE;
 	c = text;
 	if (*c) {
@@ -1713,7 +1705,7 @@ name_next (GladeXML *xml)
 			valid_name = FALSE;
 		}
 	}
-		
+
 	while (valid_name && *c) {
 		gunichar uc = g_utf8_get_char (c);
 		/* FIXME: Check this */
@@ -1722,24 +1714,24 @@ name_next (GladeXML *xml)
 			break;
 		}
 		c = g_utf8_next_char (c);
-	}		
+	}
 
 	if (valid_name) {
-		g_object_set_data_full (G_OBJECT (xml), "selected_name", 
+		g_object_set_data_full (G_OBJECT (xml), "selected_name",
 					text, g_free);
 
 		text = g_strdup (gtk_entry_get_text (GTK_ENTRY (glade_xml_get_widget (xml, "description_entry"))));
 		if (text && text[0]) {
-			g_object_set_data_full (G_OBJECT (xml), "selected_description", 
+			g_object_set_data_full (G_OBJECT (xml), "selected_description",
 						text, g_free);
 		} else if (text) {
 			g_free (text);
 		}
-		
+
 		return glade_xml_get_widget (xml, "apply_page");
 	} else {
 		GtkWidget *dialog;
-		
+
 		g_free (text);
 
 		dialog = gtk_message_dialog_new (GTK_WINDOW (glade_xml_get_widget (xml, "add_printer_window")),
@@ -1755,10 +1747,10 @@ name_next (GladeXML *xml)
 }
 
 static ipp_t *
-add_printer_request (const char *printer_name, 
+add_printer_request (const char *printer_name,
 		     const char *device_uri,
-		     PPD *ppd, 
-		     const char *human_name, 
+		     PPD *ppd,
+		     const char *human_name,
 		     const char *printer_description,
 		     GError **err)
 {
@@ -1767,12 +1759,12 @@ add_printer_request (const char *printer_name,
 
 	request = gnome_cups_request_new_for_printer (CUPS_ADD_PRINTER,
 						      printer_name);
-	ippAddString (request, 
+	ippAddString (request,
 		      IPP_TAG_PRINTER, IPP_TAG_TEXT,
 		      "printer-location", NULL, gnome_cups_strdup (device_uri));
 	if (printer_description) {
-		ippAddString (request, 
-			      IPP_TAG_PRINTER, IPP_TAG_TEXT, 
+		ippAddString (request,
+			      IPP_TAG_PRINTER, IPP_TAG_TEXT,
 			      "printer-info", NULL, gnome_cups_strdup (printer_description));
 	}
 
@@ -1791,21 +1783,21 @@ add_printer_request (const char *printer_name,
 
 static gboolean
 add_cups_printer (GladeXML *xml,
-		  const char *device_uri, PPD *ppd, 
+		  const char *device_uri, PPD *ppd,
 		  const char *printer_name,
 		  const char *printer_description)
 {
 	ipp_t *response;
 	GError *err = NULL;
 
-	response = add_printer_request (printer_name, device_uri, ppd, 
+	response = add_printer_request (printer_name, device_uri, ppd,
 					printer_name, printer_description,
 					&err);
 	ippDelete (response);
 
 	if (err) {
 		gnome_cups_error_dialog (glade_xml_get_widget (xml, "add_printer_window"),
-					 _("Couldn't add printer"), 
+					 _("Couldn't add printer"),
 					 err);
 		g_error_free (err);
 		return FALSE;
@@ -1822,7 +1814,7 @@ apply_prepare (GladeXML *xml)
 	char *text;
 	char *vendor;
 	char *model;
-	
+
 	text = get_selected_location (xml);
 	label = glade_xml_get_widget (xml, "location_label");
 	gtk_label_set_text (GTK_LABEL (label), text);
@@ -1832,21 +1824,21 @@ apply_prepare (GladeXML *xml)
 	model = get_selected_model (xml);
 	text = g_strdup_printf ("%s %s", vendor, model);
 	label = glade_xml_get_widget (xml, "type_label");
-	gtk_label_set_text (GTK_LABEL (label), text);	
+	gtk_label_set_text (GTK_LABEL (label), text);
 	g_free (text);
 	g_free (vendor);
 	g_free (model);
 
 	text = g_object_get_data (G_OBJECT (xml), "selected_name");
 	label = glade_xml_get_widget (xml, "name_label");
-	gtk_label_set_text (GTK_LABEL (label), text);	
+	gtk_label_set_text (GTK_LABEL (label), text);
 
 	text = g_object_get_data (G_OBJECT (xml), "selected_description");
 	label = glade_xml_get_widget (xml, "description_label");
 	gtk_label_set_text (GTK_LABEL (label), (text && text[0]) ? text : "None");
 
 	druid = glade_xml_get_widget (xml, "add_printer_druid");
-	gnome_druid_set_show_finish (GNOME_DRUID (druid), TRUE);	
+	gnome_druid_set_show_finish (GNOME_DRUID (druid), TRUE);
 }
 
 static GtkWidget *
@@ -1873,9 +1865,9 @@ apply_finish (GladeXML *xml)
 		g_object_unref (xml);
 		gtk_main_quit ();
 	}
-	
+
 	g_free (uri);
-	
+
 	return NULL;
 }
 
@@ -1904,7 +1896,7 @@ DruidPageDescription pages[] = {
 		network_location_back,
 		network_location_next,
 	},
-	{ 
+	{
 		"details_page",
 		details_setup,
 		NULL,
@@ -1912,7 +1904,7 @@ DruidPageDescription pages[] = {
 		details_back,
 		details_next
 	},
-	{ 
+	{
 		"name_page",
 		name_setup,
 		NULL,
@@ -1944,7 +1936,7 @@ cancel_cb (GtkWidget *widget, gpointer user_data)
 	gtk_main_quit ();
 }
 
-int 
+int
 main (int argc, char *argv[])
 {
 	GladeXML *xml;
@@ -1954,7 +1946,7 @@ main (int argc, char *argv[])
 	int base_size;
 	GdkPixbuf *pixbuf;
 
-	gnome_program_init ("gnome-cups-add", 
+	gnome_program_init ("gnome-cups-add",
 			    VERSION,
 			    LIBGNOMEUI_MODULE, argc, argv,
 			    GNOME_PROGRAM_STANDARD_PROPERTIES,
@@ -1963,16 +1955,16 @@ main (int argc, char *argv[])
 	glade_init ();
 	gnome_cups_ui_init ();
 
-	xml = glade_xml_new (GNOME_CUPS_MANAGER_DATADIR "/gnome-cups-add.glade", 
+	xml = glade_xml_new (GNOME_CUPS_MANAGER_DATADIR "/gnome-cups-add.glade",
 			     "add_printer_window",
 			     GETTEXT_PACKAGE);
-	
+
 	widget = glade_xml_get_widget (xml, "add_printer_window");
-	g_signal_connect (widget, "delete_event", 
+	g_signal_connect (widget, "delete_event",
 			  G_CALLBACK (delete_event_cb), NULL);
 
 	theme = gnome_icon_theme_new ();
-	
+
 	filename = gnome_icon_theme_lookup_icon (theme,
 						 "gnome-dev-printer-new",
 						 32,
@@ -1980,10 +1972,12 @@ main (int argc, char *argv[])
 						 &base_size);
 	g_object_unref (theme);
 
-	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
-
-	gtk_window_set_icon (GTK_WINDOW (widget), pixbuf);
-	g_object_unref (pixbuf);
+	if (filename != NULL) {
+		pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+		gtk_window_set_icon (GTK_WINDOW (widget), pixbuf);
+		g_object_unref (pixbuf);
+	} else
+		g_warning ("unable to load icon 'gnome-dev-printer-new'");
 
 	setup_ppds (xml);
 
@@ -1992,7 +1986,7 @@ main (int argc, char *argv[])
 	gtk_widget_show (widget);
 
 	widget = glade_xml_get_widget (xml, "add_printer_druid");
-	g_signal_connect (widget, "cancel", 
+	g_signal_connect (widget, "cancel",
 			  G_CALLBACK (cancel_cb), NULL);
 
 	gtk_main ();
