@@ -122,8 +122,8 @@ get_listener (gboolean *is_new)
 		*is_new = TRUE;
 		ret = BONOBO_OBJREF (listener);
 
-		reg_result = bonobo_activation_active_server_register
-			("OAFIID:gnome-printer-view", ret);
+		reg_result = bonobo_activation_register_active_server
+			("OAFIID:gnome-printer-view", ret, NULL);
 		switch (reg_result) {
 		case Bonobo_ACTIVATION_REG_SUCCESS :
 			break;
